/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.client.InstanceDetails;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.UIObject;

import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementException;
import org.ow2.orchestra.console.GWTConsole.client.Commons.IndexFocusPanel;
import org.ow2.orchestra.console.GWTConsole.client.Event.RefreshDoneEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.SuspendTimerEvent;
import org.ow2.orchestra.console.GWTConsole.client.ListInstance.InstanceGrid;
import org.ow2.orchestra.console.GWTConsole.client.ListInstance.MInstance;
import org.ow2.orchestra.console.GWTConsole.client.Popup.PPopup;
import org.ow2.orchestra.console.GWTConsole.client.Tab.PTab;


/**
 * Presenter of the Instance Details class
 *
 * @author Jean-Yves Goubet
 *
 */
public class InstanceDetailsGrid extends InstanceGrid {

  private String instanceId = "";

  /**
   * Constructor of the presenter
   *
   * @author Jean-Yves Goubet
   * @param view : the view of the Instance Details class
   * @param instanceId
   */
  public InstanceDetailsGrid() {
    super(false);
  }

  @Override
  protected void onEnsureDebugId(final String baseID) {
    super.onEnsureDebugId(baseID);

    this.getWidget(0, 0).ensureDebugId("instanceIdLabel-UUID-" + this.instanceId + "-IA");
    this.getWidget(0, 1).ensureDebugId("processIdLabel-UUID-" + this.instanceId + "-IA");
    this.getWidget(0, 2).ensureDebugId("processNameLabel-UUID-" + this.instanceId + "-IA");
    this.getWidget(0, 3).ensureDebugId("instanceStateLabel-UUID-" + this.instanceId + "-IA");
    this.getWidget(0, 4).ensureDebugId("instanceStartDateLabel-UUID-" + this.instanceId + "-IA");
    this.getWidget(0, 5).ensureDebugId("instanceLastUpdateDateLabel-UUID-" + this.instanceId + "-IA");
    this.getWidget(0, 6).ensureDebugId("instanceActionsLabel-UUID-" + this.instanceId + "-IA");

    this.getWidget(1, 0).ensureDebugId("instanceIdValue-UUID-" + this.instanceId + "-IA");
    this.getWidget(1, 1).ensureDebugId("processIdValue-UUID-" + this.instanceId + "-IA");
    this.getWidget(1, 2).ensureDebugId("processNameValue-UUID-" + this.instanceId + "-IA");
    this.getWidget(1, 3).ensureDebugId("instanceStateValue-UUID-" + this.instanceId + "-IA");
    this.getWidget(1, 4).ensureDebugId("instanceStartDateValue-UUID-" + this.instanceId + "-IA");
    this.getWidget(1, 5).ensureDebugId("instanceLastUpdateDateValue-UUID-" + this.instanceId + "-IA");
    this.getWidget(1, 6).ensureDebugId("instanceActionsValue-UUID-" + this.instanceId + "-IA");

    UIObject.ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(1, 6)).getWidget()).
        getWidget(0).getElement(), "activitiesImageID-UUID-" + this.instanceId + "-IA");
    UIObject.ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(1, 6)).getWidget()).
        getWidget(1).getElement(), "suspendImageID-UUID-" + this.instanceId + "-IA");
    UIObject.ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(1, 6)).getWidget()).
        getWidget(2).getElement(), "resumeImageID-UUID-" + this.instanceId + "-IA");
    UIObject.ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(1, 6)).getWidget()).
        getWidget(3).getElement(), "exitImageID-UUID-" + this.instanceId + "-IA");
    UIObject.ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(1, 6)).getWidget()).
        getWidget(4).getElement(), "removeImageID-UUID-" + this.instanceId + "-IA");
  }


  /**
   * This method refresh the info about the instance
   *
   * @author Jean-Yves Goubet
   */
  @Override
  public void refreshCurrentList() {
    GWT.log("InstanceDetailsGrid - start call");

    this.orchestramanagement.getInstance(this.instanceId, new AsyncCallback<MInstance>() {
      public void onSuccess(final MInstance instance) {
        // check that the result list contains 1 element
        InstanceDetailsGrid.this.manageGrid();
        InstanceDetailsGrid.this.addInstance(instance);
        InstanceDetailsGrid.this.eventBus.fireEvent(new RefreshDoneEvent());
      }

      public void onFailure(final Throwable caught) {
        if (InstanceDetailsGrid.this.presenterTabLayoutPanel.tabAlreadyExists(InstanceDetailsGrid.this.instanceId, PTab.Type.LIST_INSTANCE_ACTIVITY) != -1) {
          GWT.log("InstanceDetailsGrid - refreshCurrentList : onFailure : " + caught.getMessage());
          InstanceDetailsGrid.this.eventBus.fireEvent(new SuspendTimerEvent(true));
          if (caught instanceof OrchestraManagementException) {
            PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(),
                ((OrchestraManagementException)caught).getDetails());
          } else {
            PPopup.launch(PPopup.PopupType.ErrorPopup, "Unable to access to Orchestra !", caught.getMessage());
          }
        }
      }
    });

  }

  public void setInstanceId(final String instanceId) {
    this.instanceId = instanceId;
  }

}
