/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.client.ListInstanceActivity;

import com.google.gwt.event.shared.HandlerManager;

import org.ow2.orchestra.console.GWTConsole.client.Commons.PresenterIF;
import org.ow2.orchestra.console.GWTConsole.client.Event.ViewRefreshEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.ViewRefreshHandler;
import org.ow2.orchestra.console.GWTConsole.client.Tab.PTab;
import org.ow2.orchestra.console.GWTConsole.client.TabLayoutPanel.PTabLayoutPanel;


/**
 * Presenter of the instance activity view
 *
 * @author Jean-Yves Goubet
 *
 */
public class PListInstanceActivity extends PresenterIF {

  private VListInstanceActivityIF view;
  private String instanceId = null;

  private ViewRefreshHandler refreshHandler = null;
  private HandlerManager eventBus = null;
  private boolean removed = false;


  public PListInstanceActivity(final PTabLayoutPanel presenterTabLayoutPanel,
      final String instanceId, final String instanceName, final HandlerManager eventBus) {
    //
    final VListInstanceActivity vListInstanceActivity = new VListInstanceActivity();
    this.bindingView(vListInstanceActivity);
    this.instanceId  = instanceId;

    //
    this.eventBus = eventBus;
    this.addListeners();

    this.view.getInstanceActivityGrid().setHandlerManager(eventBus);
    this.view.getInstanceDetailsGrid().setHandlerManager(eventBus);
    //
    this.view.getInstanceDetailsGrid().setInstanceId(instanceId);
    this.view.getInstanceDetailsGrid().setPresenterTabLayoutPanel(presenterTabLayoutPanel);
    //
    this.view.getInstanceActivityGrid().setInstanceId(instanceId);
    this.view.getInstanceActivityGrid().setPresenterTabLayoutPanel(presenterTabLayoutPanel);
    this.view.getInstanceActivityGrid().setStringLimits(10, 20, "...");
    //
    presenterTabLayoutPanel.addTab(instanceId, instanceName, PTab.Type.LIST_INSTANCE_ACTIVITY, vListInstanceActivity, this);

    //
    this.refresh();
  }

  @Override
  protected void bindingView(final Object view) {
    this.view = (VListInstanceActivityIF) view;
  }

  @Override
  protected void addListeners() {
    this.addViewRefreshHandlerToBus();
  }

  private void addViewRefreshHandlerToBus() {

    this.refreshHandler = new ViewRefreshHandler() {
      public void onRefresh(final ViewRefreshEvent viewRefreshEvent) {
        if (PListInstanceActivity.this.instanceId.equals(viewRefreshEvent.getElementId())
            && viewRefreshEvent.getElementType() == PTab.Type.LIST_INSTANCE_ACTIVITY) {
          PListInstanceActivity.this.refresh();
        }
      }
    };

    this.eventBus.addHandler(ViewRefreshEvent.TYPE, this.refreshHandler);
  }

  public void refresh() {

    // refresh details info
    this.view.getInstanceDetailsGrid().refreshCurrentList();
    // refresh list activity grid
    this.view.getInstanceActivityGrid().refreshCurrentList();
  }

  @Override
  protected void setDebugID() {
    // no debug id
  }


  /**
   * @return the removed
   */
  public boolean isRemoved() {
    return this.removed;
  }

  @Override
  public void destroy() {
    if (this.refreshHandler != null) {
      if (this.eventBus != null) {
        this.eventBus.removeHandler(ViewRefreshEvent.TYPE, this.refreshHandler);
        this.removed  = true;
      }
    }
  }
}
