/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.client.ListProcessActivity;

import com.google.gwt.event.shared.HandlerManager;

import org.ow2.orchestra.console.GWTConsole.client.Commons.PresenterIF;
import org.ow2.orchestra.console.GWTConsole.client.Event.ViewRefreshEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.ViewRefreshHandler;
import org.ow2.orchestra.console.GWTConsole.client.Tab.PTab;
import org.ow2.orchestra.console.GWTConsole.client.TabLayoutPanel.PTabLayoutPanel;


/**
 * Presenter of the process activity view
 *
 * @author Jean-Yves Goubet
 *
 */
public class PListProcessActivity extends PresenterIF {

  private VListProcessActivityIF view;

  protected String processId = null;
  private ViewRefreshHandler refreshHandler = null;
  private HandlerManager eventBus = null;

  public PListProcessActivity(final PTabLayoutPanel pTabLayoutPanel, final String processId, final String processName, final HandlerManager eventBus) {
    //
    final VListProcessActivity vListProcessActivity = new VListProcessActivity();
    this.bindingView(vListProcessActivity);
    this.view.getProcessDetailsGrid().setHandlerManager(eventBus);
    this.view.getProcessActivityGrid().setHandlerManager(eventBus);
    this.processId = processId;

    //
    this.eventBus = eventBus;
    this.addListeners();

    //
    this.view.getProcessDetailsGrid().setProcessId(processId);
    this.view.getProcessDetailsGrid().setStringLimitSize(10, 15, 50, "...");
    this.view.getProcessDetailsGrid().setPresenterTabLayoutPanel(pTabLayoutPanel);
    // fill the process activity grid
    this.view.getProcessActivityGrid().setProcessId(processId);
    //
    pTabLayoutPanel.addTab(processId, processName, PTab.Type.LIST_PROCESS_ACTIVITY, vListProcessActivity, this);
    //
    this.view.getProcessActivityGrid().refreshCurrentList();
    this.view.getProcessDetailsGrid().refreshCurrentList();
  }

  @Override
  protected void bindingView(final Object view) {
    this.view = (VListProcessActivityIF) view;
  }

  @Override
  protected void addListeners() {
    this.addViewRefreshHandlerToBus();
  }

  private void addViewRefreshHandlerToBus() {

    this.refreshHandler = new ViewRefreshHandler() {
      public void onRefresh(final ViewRefreshEvent viewRefreshEvent) {
        if (PListProcessActivity.this.processId.equals(viewRefreshEvent.getElementId())
            && viewRefreshEvent.getElementType() == PTab.Type.LIST_PROCESS_ACTIVITY) {
          PListProcessActivity.this.view.getProcessDetailsGrid().refreshCurrentList();
        }
      }
    };
    this.eventBus.addHandler(ViewRefreshEvent.TYPE, this.refreshHandler);
  }

  @Override
  protected void setDebugID() {
    // no debug id
  }


  @Override
  public void destroy() {
    if (this.refreshHandler != null) {
      if (this.eventBus != null) {
System.out.println("PListProcessActivity - handler delete");

        this.eventBus.removeHandler(ViewRefreshEvent.TYPE, this.refreshHandler);
      }

    }

  }

}
