/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.client.ProcessDetails;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.UIObject;

import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementException;
import org.ow2.orchestra.console.GWTConsole.client.Commons.IndexFocusPanel;
import org.ow2.orchestra.console.GWTConsole.client.Event.RefreshDoneEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.SuspendTimerEvent;
import org.ow2.orchestra.console.GWTConsole.client.ListProcess.MProcess;
import org.ow2.orchestra.console.GWTConsole.client.ListProcess.ProcessGrid;
import org.ow2.orchestra.console.GWTConsole.client.Popup.PPopup;


/**
 * Presenter of the Process Details class
 *
 * @author Jean-Yves Goubet
 *
 */
public class ProcessDetailsGrid extends ProcessGrid {

  private String processId = "";
  /**
   * Constructor of the presenter
   *
   * @author Jean-Yves Goubet
   * @param view : the view of the Process Details class
   */
  public ProcessDetailsGrid() {
    super(false);
  }

  public void setProcessId(final String processId) {
    this.processId = processId;
  }

  @Override
  protected void onEnsureDebugId(final String baseID) {
    super.onEnsureDebugId(baseID);

    this.getWidget(0,0).ensureDebugId("processIdLabel-UUID-" + this.processId + "-PA");
    this.getWidget(0,1).ensureDebugId("processNameLabel-UUID-" + this.processId + "-PA");
    this.getWidget(0,2).ensureDebugId("processNamespaceLabel-UUID-" + this.processId + "-PA");
    this.getWidget(0,3).ensureDebugId("processVersionLabel-UUID-" + this.processId + "-PA");
    this.getWidget(0,4).ensureDebugId("processDeploymentDateLabel-UUID-" + this.processId + "-PA");
    this.getWidget(0,5).ensureDebugId("processUndeploymentDateLabel-UUID-" + this.processId + "-PA");
    this.getWidget(0,6).ensureDebugId("processStateLabel-UUID-" + this.processId + "-PA");
    this.getWidget(0,7).ensureDebugId("processActionsLabel-UUID-" + this.processId + "-PA");


    this.getWidget(1,0).ensureDebugId("processIdValue-UUID-" + this.processId + "-PA");
    this.getWidget(1,1).ensureDebugId("processNameValue-UUID-" + this.processId + "-PA");
    this.getWidget(1,2).ensureDebugId("processNamespaceValue-UUID-" + this.processId + "-PA");
    this.getWidget(1,3).ensureDebugId("processVersionValue-UUID-" + this.processId + "-PA");
    this.getWidget(1,4).ensureDebugId("processDeploymentDateValue-UUID-" + this.processId + "-PA");
    this.getWidget(1,5).ensureDebugId("processUndeploymentDateValue-UUID-" + this.processId + "-PA");
    this.getWidget(1,6).ensureDebugId("processStateValue-UUID-" + this.processId + "-PA");
    this.getWidget(1,7).ensureDebugId("processActionsValue-UUID-" + this.processId + "-PA");

    UIObject.ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(1, 7)).getWidget()).
        getWidget(0).getElement(), "instancesImageID-UUID-" + this.processId + "-PA");
    UIObject.ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(1, 7)).getWidget()).
        getWidget(1).getElement(), "activitiesImageID-UUID-" + this.processId + "-PA");
    UIObject.ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(1, 7)).getWidget()).
        getWidget(2).getElement(), "retireImageID-UUID-" + this.processId + "-PA");
    UIObject.ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(1, 7)).getWidget()).
        getWidget(3).getElement(), "activateImageID-UUID-" + this.processId + "-PA");
    UIObject.ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(1, 7)).getWidget()).
        getWidget(4).getElement(), "undeployImageID-UUID-" + this.processId + "-PA");
    UIObject.ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(1, 7)).getWidget()).
        getWidget(5).getElement(), "removeImageID-UUID-" + this.processId + "-PA");
  }

  /**
   * To get info on the process from Orchestra server.
   *
   * @author Jean-Yves Goubet
   */
  @Override
  public void refreshCurrentList() {
//System.out.println("ProcessDetailsGrid - processId = " + processId);

    this.orchestramanagement.getProcess(this.processId, new AsyncCallback<MProcess>() {
      public void onSuccess(final MProcess process) {
        ProcessDetailsGrid.this.manageGrid();
        ProcessDetailsGrid.this.addProcess(process);
        ProcessDetailsGrid.this.eventBus.fireEvent(new RefreshDoneEvent());
      }

      public void onFailure(final Throwable caught) {
        GWT.log("ProcessDetailsGrid - refreshCurrentList: onFailure : " + caught.getMessage());
        ProcessDetailsGrid.this.eventBus.fireEvent(new SuspendTimerEvent(true));
        if (caught instanceof OrchestraManagementException) {
          PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(),
              ((OrchestraManagementException)caught).getDetails());
        } else {
          PPopup.launch(PPopup.PopupType.ErrorPopup, "Unable to access to Orchestra !", caught.getMessage());
        }
      }
    });

  }

}
